@@ -816,6 +816,7 @@ get_groups_cb (GDataService *service,
 	}
 
 	finish_operation (backend, -2, gdata_error);
+	g_object_unref (backend);
 
 	g_clear_error (&gdata_error);
 }
@@ -839,6 +840,8 @@ get_groups (EBookBackend *backend)
 		gdata_contacts_query_set_show_deleted (GDATA_CONTACTS_QUERY (query), TRUE);
 	}
 
+	g_object_ref (backend);
+
 	/* Run the query asynchronously */
 	cancellable = start_operation (backend, -2, NULL, _("Querying for updated groups…"));
 	gdata_contacts_service_query_groups_async (
@@ -1018,12 +1021,8 @@ request_authorization (EBookBackend *backend,
                        GError **error)
 {
 	EBookBackendGooglePrivate *priv;
-	ESourceRegistry *registry;
-	ESource *source;
 
 	priv = E_BOOK_BACKEND_GOOGLE_GET_PRIVATE (backend);
-	source = e_backend_get_source (E_BACKEND (backend));
-	registry = e_book_backend_get_registry (backend);
 
 	/* Make sure we have the GDataService configured
 	 * before requesting authorization. */
@@ -1070,8 +1069,9 @@ request_authorization (EBookBackend *backend,
 #endif
 
 	/* Otherwise it's up to us to obtain a login secret. */
-	return e_source_registry_authenticate_sync (
-		registry, source, E_SOURCE_AUTHENTICATOR (backend),
+	return e_backend_authenticate_sync (
+		E_BACKEND (backend),
+		E_SOURCE_AUTHENTICATOR (backend),
 		cancellable, error);
 }
 
