@@ -20,6 +20,10 @@
  * Author: Chris Toshok (toshok@ximian.com)
  */
 
+#if !defined (__LIBEBOOK_H_INSIDE__) && !defined (LIBEBOOK_COMPILATION)
+#error "Only <libebook/libebook.h> should be included directly."
+#endif
+
 #ifndef _EVCARD_H
 #define _EVCARD_H
 
@@ -111,7 +115,7 @@ G_BEGIN_DECLS
 #define EVC_X_GOOGLE_TALK		"X-GOOGLE-TALK"
 
 /**
- * EVC_X_TWITTER
+ * EVC_X_TWITTER:
  * Twitter name(s).
  *
  * Since: 3.6
@@ -131,12 +135,12 @@ G_BEGIN_DECLS
 #define EVC_X_VIDEO_URL			"X-EVOLUTION-VIDEO-URL"
 #define EVC_X_WANTS_HTML		"X-MOZILLA-HTML"
 #define EVC_X_YAHOO			"X-YAHOO"
-#define EVC_X_BOOK_URI			"X-EVOLUTION-BOOK-URI"
+#define EVC_X_BOOK_UID			"X-EVOLUTION-BOOK-UID"
 
 /* Constants for Evo contact lists only */
 
 /**
- * EVC_CONTACT_LIST
+ * EVC_CONTACT_LIST:
  *
  * Since: 3.2
  **/
