@@ -77,6 +77,23 @@ typedef enum _camel_imapx_id_t {
 	IMAPX_VANISHED,
 } camel_imapx_id_t;
 
+#define CAMEL_IMAPX_UNTAGGED_BAD        "BAD"
+#define CAMEL_IMAPX_UNTAGGED_BYE        "BYE"
+#define CAMEL_IMAPX_UNTAGGED_CAPABILITY "CAPABILITY"
+#define CAMEL_IMAPX_UNTAGGED_EXISTS     "EXISTS"
+#define CAMEL_IMAPX_UNTAGGED_EXPUNGE    "EXPUNGE"
+#define CAMEL_IMAPX_UNTAGGED_FETCH      "FETCH"
+#define CAMEL_IMAPX_UNTAGGED_FLAGS      "FLAGS"
+#define CAMEL_IMAPX_UNTAGGED_LIST       "LIST"
+#define CAMEL_IMAPX_UNTAGGED_LSUB       "LSUB"
+#define CAMEL_IMAPX_UNTAGGED_NAMESPACE  "NAMESPACE"
+#define CAMEL_IMAPX_UNTAGGED_NO         "NO"
+#define CAMEL_IMAPX_UNTAGGED_OK         "OK"
+#define CAMEL_IMAPX_UNTAGGED_PREAUTH    "PREAUTH"
+#define CAMEL_IMAPX_UNTAGGED_RECENT     "RECENT"
+#define CAMEL_IMAPX_UNTAGGED_STATUS     "STATUS"
+#define CAMEL_IMAPX_UNTAGGED_VANISHED   "VANISHED"
+
 /* str MUST be in upper case, tokenised using gperf function */
 camel_imapx_id_t imapx_tokenise (register const gchar *str, register guint len);
 
@@ -118,6 +135,7 @@ struct _capability_info {
 
 struct _capability_info *imapx_parse_capability (struct _CamelIMAPXStream *stream, GCancellable *cancellable, GError **error);
 void imapx_free_capability (struct _capability_info *);
+guint32 imapx_register_capability (const gchar *capability);
 
 gboolean imapx_parse_param_list (struct _CamelIMAPXStream *is, struct _camel_header_param **plist, GCancellable *cancellable, GError **error) /* IO,PARSE */;
 struct _CamelContentDisposition *imapx_parse_ext_optional (struct _CamelIMAPXStream *is, GCancellable *cancellable, GError **error) /* IO,PARSE */;
