@@ -136,7 +136,8 @@ camel_offline_store_set_online_sync (CamelOfflineStore *store,
 	/* Returning to online mode is the simpler case. */
 	if (!store_is_online) {
 		store->priv->online = online;
-		return camel_service_connect_sync (service, error);
+		return camel_service_connect_sync (
+			service, cancellable, error);
 	}
 
 	/* network available -> network unavailable */
@@ -172,7 +173,7 @@ camel_offline_store_set_online_sync (CamelOfflineStore *store,
 	}
 
 	success = camel_service_disconnect_sync (
-		service, network_available, error);
+		service, network_available, cancellable, error);
 
 	store->priv->online = online;
 
