@@ -42,8 +42,14 @@
 /* key index debug */
 #define k(x) /*(printf ("%s (%d):%s: ",  __FILE__, __LINE__, __PRETTY_FUNCTION__),(x))*/
 
-#define CAMEL_PARTITION_TABLE_LOCK(kf, lock) (g_static_mutex_lock(&(kf)->priv->lock))
-#define CAMEL_PARTITION_TABLE_UNLOCK(kf, lock) (g_static_mutex_unlock(&(kf)->priv->lock))
+#define CAMEL_PARTITION_TABLE_LOCK(kf, lock) \
+	(g_static_mutex_lock (&(kf)->priv->lock))
+#define CAMEL_PARTITION_TABLE_UNLOCK(kf, lock) \
+	(g_static_mutex_unlock (&(kf)->priv->lock))
+
+#define CAMEL_PARTITION_TABLE_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_PARTITION_TABLE, CamelPartitionTablePrivate))
 
 struct _CamelPartitionTablePrivate {
 	GStaticMutex lock;	/* for locking partition */
@@ -87,8 +93,7 @@ camel_partition_table_class_init (CamelPartitionTableClass *class)
 static void
 camel_partition_table_init (CamelPartitionTable *cpi)
 {
-	cpi->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		cpi, CAMEL_TYPE_PARTITION_TABLE, CamelPartitionTablePrivate);
+	cpi->priv = CAMEL_PARTITION_TABLE_GET_PRIVATE (cpi);
 
 	g_queue_init (&cpi->partition);
 	g_static_mutex_init (&cpi->priv->lock);
@@ -612,8 +617,14 @@ fail:
 
 /* ********************************************************************** */
 
-#define CAMEL_KEY_TABLE_LOCK(kf, lock) (g_static_mutex_lock(&(kf)->priv->lock))
-#define CAMEL_KEY_TABLE_UNLOCK(kf, lock) (g_static_mutex_unlock(&(kf)->priv->lock))
+#define CAMEL_KEY_TABLE_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_KEY_TABLE, CamelKeyTablePrivate))
+
+#define CAMEL_KEY_TABLE_LOCK(kf, lock) \
+	(g_static_mutex_lock (&(kf)->priv->lock))
+#define CAMEL_KEY_TABLE_UNLOCK(kf, lock) \
+	(g_static_mutex_unlock (&(kf)->priv->lock))
 
 struct _CamelKeyTablePrivate {
 	GStaticMutex lock;	/* for locking key */
@@ -655,8 +666,7 @@ camel_key_table_class_init (CamelKeyTableClass *class)
 static void
 camel_key_table_init (CamelKeyTable *table)
 {
-	table->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		table, CAMEL_TYPE_KEY_TABLE, CamelKeyTablePrivate);
+	table->priv = CAMEL_KEY_TABLE_GET_PRIVATE (table);
 	g_static_mutex_init (&table->priv->lock);
 }
 
