@@ -61,7 +61,7 @@ static CamelProvider vee_provider = {
 
 	"vfolder",
 
-	CAMEL_PROVIDER_IS_STORAGE,
+	CAMEL_PROVIDER_IS_STORAGE | CAMEL_PROVIDER_IS_LOCAL,
 	CAMEL_URL_NEED_PATH | CAMEL_URL_PATH_IS_ABSOLUTE | CAMEL_URL_FRAGMENT_IS_PATH,
 
 	NULL,	/* extra conf */
@@ -337,11 +337,19 @@ camel_provider_list (gboolean load)
 		g_hash_table_foreach (module_table, add_to_list, &list);
 		for (w = list; w; w = w->next) {
 			CamelProviderModule *m = w->data;
+			GError *error = NULL;
 
 			if (!m->loaded) {
-				camel_provider_load (m->path, NULL);
+				camel_provider_load (m->path, &error);
 				m->loaded = 1;
 			}
+
+			if (error != NULL) {
+				g_critical (
+					"%s: %s", G_STRFUNC,
+					error->message);
+				g_error_free (error);
+			}
 		}
 		g_list_free (list);
 		list = NULL;
