@@ -69,7 +69,8 @@ camel_url_get_type (void)
  * Returns: a parsed #CamelURL
  **/
 CamelURL *
-camel_url_new_with_base (CamelURL *base, const gchar *url_string)
+camel_url_new_with_base (CamelURL *base,
+                         const gchar *url_string)
 {
 	CamelURL *url;
 	const gchar *end, *hash, *colon, *semi, *at, *slash, *question;
@@ -99,7 +100,7 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 
 	/* Find protocol: initial [a-z+.-]* substring until ":" */
 	p = url_string;
-	while (p < end && (isalnum ((guchar)*p) ||
+	while (p < end && (isalnum ((guchar) * p) ||
 			   *p == '.' || *p == '+' || *p == '-'))
 		p++;
 
@@ -128,11 +129,9 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 		if (at && at < slash) {
 			colon = strchr (url_string, ':');
 			if (colon && colon < at) {
-				url->passwd = g_strndup (colon + 1,
-							 at - colon - 1);
-				camel_url_decode (url->passwd);
+				/* XXX We used to extract and store the
+				 *     password here, now we just eat it. */
 			} else {
-				url->passwd = NULL;
 				colon = at;
 			}
 
@@ -151,7 +150,7 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 			camel_url_decode (url->user);
 			url_string = at + 1;
 		} else
-			url->user = url->passwd = url->authmech = NULL;
+			url->user = url->authmech = NULL;
 
 		/* Find host and port. */
 		colon = strchr (url_string, ':');
@@ -216,7 +215,7 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 	if (base && !url->protocol && url->host)
 		url->protocol = g_strdup (base->protocol);
 	else if (base && !url->protocol) {
-		if (!url->user && !url->authmech && !url->passwd &&
+		if (!url->user && !url->authmech &&
 		    !url->host && !url->port && !url->path &&
 		    !url->params && !url->query && !url->fragment)
 			url->fragment = g_strdup (base->fragment);
@@ -224,7 +223,6 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 		url->protocol = g_strdup (base->protocol);
 		url->user = g_strdup (base->user);
 		url->authmech = g_strdup (base->authmech);
-		url->passwd = g_strdup (base->passwd);
 		url->host = g_strdup (base->host);
 		url->port = base->port;
 
@@ -240,7 +238,7 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 			gchar *newpath, *last, *p, *q;
 
 			/* the base->path is NULL if given Content-Base url was without last slash,
-			   i.e. like "http://example.com" (this expected only "http://example.com/") */
+			 * i.e. like "http://example.com" (this expected only "http://example.com/") */
 			last = base->path ? strrchr (base->path, '/') : NULL;
 			if (last) {
 				newpath = g_strdup_printf ("%.*s/%s",
@@ -296,7 +294,9 @@ camel_url_new_with_base (CamelURL *base, const gchar *url_string)
 }
 
 static void
-copy_param (GQuark key_id, gpointer data, gpointer user_data)
+copy_param (GQuark key_id,
+            gpointer data,
+            gpointer user_data)
 {
 	GData **copy = user_data;
 
@@ -343,7 +343,8 @@ camel_url_new (const gchar *url_string,
  * Returns: a string representing @url, which the caller must free
  **/
 gchar *
-camel_url_to_string (CamelURL *url, CamelURLFlags flags)
+camel_url_to_string (CamelURL *url,
+                     CamelURLFlags flags)
 {
 	GString *str;
 	gchar *return_result;
@@ -372,10 +373,6 @@ camel_url_to_string (CamelURL *url, CamelURLFlags flags)
 				g_string_append (str, ";auth=");
 				append_url_encoded (str, url->authmech, ":@/");
 			}
-			if (url->passwd && !(flags & CAMEL_URL_HIDE_PASSWORD)) {
-				g_string_append_c (str, ':');
-				append_url_encoded (str, url->passwd, "@/");
-			}
 			g_string_append_c (str, '@');
 		}
 		append_url_encoded (str, url->host, ":/");
@@ -405,7 +402,9 @@ camel_url_to_string (CamelURL *url, CamelURLFlags flags)
 }
 
 static void
-output_param (GQuark key_id, gpointer data, gpointer user_data)
+output_param (GQuark key_id,
+              gpointer data,
+              gpointer user_data)
 {
 	GString *str = user_data;
 
@@ -427,8 +426,6 @@ void
 camel_url_free (CamelURL *url)
 {
 	if (url) {
-		if (url->passwd)
-			memset (url->passwd, 0, strlen (url->passwd));
 		if (url->user)
 			memset (url->user, 0, strlen (url->user));
 		if (url->host)
@@ -436,7 +433,6 @@ camel_url_free (CamelURL *url)
 		g_free (url->protocol);
 		g_free (url->user);
 		g_free (url->authmech);
-		g_free (url->passwd);
 		g_free (url->host);
 		g_free (url->path);
 		g_datalist_clear (&url->params);
@@ -498,23 +494,6 @@ camel_url_set_authmech (CamelURL *url,
 	url->authmech = g_strdup (authmech);
 }
 
-/**
- * camel_url_set_passwd:
- * @url: a #CamelURL
- * @passwd: password
- *
- * Set the password of a #CamelURL.
- **/
-void
-camel_url_set_passwd (CamelURL *url,
-                      const gchar *passwd)
-{
-	g_return_if_fail (url != NULL);
-
-	g_free (url->passwd);
-	url->passwd = g_strdup (passwd);
-}
-
 /**
  * camel_url_set_host:
  * @url: a #CamelURL
@@ -591,7 +570,8 @@ camel_url_set_fragment (CamelURL *url,
  * Set the port on a #CamelURL.
  **/
 void
-camel_url_set_port (CamelURL *url, gint port)
+camel_url_set_port (CamelURL *url,
+                    gint port)
 {
 	g_return_if_fail (url != NULL);
 
@@ -607,7 +587,9 @@ camel_url_set_port (CamelURL *url, gint port)
  * Set a param on the #CamelURL.
  **/
 void
-camel_url_set_param (CamelURL *url, const gchar *name, const gchar *value)
+camel_url_set_param (CamelURL *url,
+                     const gchar *name,
+                     const gchar *value)
 {
 	g_return_if_fail (url != NULL);
 
@@ -627,7 +609,8 @@ camel_url_set_param (CamelURL *url, const gchar *name, const gchar *value)
  * Returns: the value of a param if found or %NULL otherwise
  **/
 const gchar *
-camel_url_get_param (CamelURL *url, const gchar *name)
+camel_url_get_param (CamelURL *url,
+                     const gchar *name)
 {
 	g_return_val_if_fail (url != NULL, NULL);
 
@@ -655,7 +638,9 @@ static const gchar url_encoded_char[] = {
 };
 
 static void
-append_url_encoded (GString *str, const gchar *in, const gchar *extra_enc_chars)
+append_url_encoded (GString *str,
+                    const gchar *in,
+                    const gchar *extra_enc_chars)
 {
 	const guchar *s = (const guchar *) in;
 
@@ -680,7 +665,8 @@ append_url_encoded (GString *str, const gchar *in, const gchar *extra_enc_chars)
  * Returns: the encoded string
  **/
 gchar *
-camel_url_encode (const gchar *part, const gchar *escape_extra)
+camel_url_encode (const gchar *part,
+                  const gchar *escape_extra)
 {
 	GString *str;
 	gchar *encoded;
@@ -742,7 +728,8 @@ camel_url_hash (gconstpointer v)
 }
 
 static gint
-check_equal (gchar *s1, gchar *s2)
+check_equal (gchar *s1,
+             gchar *s2)
 {
 	if (s1 == NULL) {
 		if (s2 == NULL)
@@ -758,7 +745,8 @@ check_equal (gchar *s1, gchar *s2)
 }
 
 gint
-camel_url_equal (gconstpointer v, gconstpointer v2)
+camel_url_equal (gconstpointer v,
+                 gconstpointer v2)
 {
 	const CamelURL *u1 = v, *u2 = v2;
 
@@ -790,7 +778,6 @@ camel_url_copy (CamelURL *in)
 	out->protocol = g_strdup (in->protocol);
 	out->user = g_strdup (in->user);
 	out->authmech = g_strdup (in->authmech);
-	out->passwd = g_strdup (in->passwd);
 	out->host = g_strdup (in->host);
 	out->port = in->port;
 	out->path = g_strdup (in->path);
@@ -807,28 +794,26 @@ gchar *
 camel_url_decode_path (const gchar *path)
 {
 	gchar **comps;
-	gchar *new_path = NULL;
 	GString *str;
-	gint i = 0;
+	guint length, ii;
 
-	if (!path)
-                return g_strdup("");    /* ??? or NULL? */
+	if (path == NULL || *path == '\0')
+		return g_strdup ("");    /* ??? or NULL? */
 
 	str = g_string_new (NULL);
 
-        comps = g_strsplit (path, "/", -1);
-	while (comps[i]) {
-		camel_url_decode (comps[i]);
-		g_string_append (str, comps[i]);
-		g_string_append_c (str, '/');
-		i++;
+	comps = g_strsplit (path, "/", -1);
+	length = g_strv_length (comps);
+
+	for (ii = 0; ii < length; ii++) {
+		if (ii > 0)
+			g_string_append_c (str, '/');
+		camel_url_decode (comps[ii]);
+		g_string_append (str, comps[ii]);
 	}
 
-        /* Strip-off the trailing "/" */
-	new_path = g_strndup (str->str, str->len-1);
-
 	g_strfreev (comps);
-	g_string_free (str, TRUE);
 
-	return new_path;
+	return g_string_free (str, FALSE);
 }
+
