@@ -506,7 +506,7 @@ camel_imap_settings_class_init (CamelImapSettingsClass *class)
 			"namespace",
 			"Namespace",
 			"Custom IMAP namespace",
-			NULL,
+			"",
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
@@ -632,6 +632,9 @@ camel_imap_settings_init (CamelImapSettings *settings)
 {
 	settings->priv = CAMEL_IMAP_SETTINGS_GET_PRIVATE (settings);
 	settings->priv->property_lock = g_mutex_new ();
+
+	/* The default namespace is an empty string. */
+	settings->priv->namespace = g_strdup ("");
 }
 
 /**
