@@ -41,6 +41,9 @@
 
 #include "e-collection-backend.h"
 
+#include <config.h>
+#include <glib/gi18n-lib.h>
+
 #include <libedataserver/libedataserver.h>
 
 #include <libebackend/e-server-side-source.h>
@@ -52,7 +55,10 @@
 
 struct _ECollectionBackendPrivate {
 	GWeakRef server;
-	GQueue children;
+
+	/* Set of ESources */
+	GHashTable *children;
+	GMutex *children_lock;
 
 	gchar *cache_dir;
 
@@ -82,6 +88,49 @@ G_DEFINE_ABSTRACT_TYPE (
 	e_collection_backend,
 	E_TYPE_BACKEND)
 
+static void
+collection_backend_children_insert (ECollectionBackend *backend,
+                                    ESource *source)
+{
+	g_mutex_lock (backend->priv->children_lock);
+
+	g_hash_table_add (backend->priv->children, g_object_ref (source));
+
+	g_mutex_unlock (backend->priv->children_lock);
+}
+
+static gboolean
+collection_backend_children_remove (ECollectionBackend *backend,
+                                    ESource *source)
+{
+	gboolean removed;
+
+	g_mutex_lock (backend->priv->children_lock);
+
+	removed = g_hash_table_remove (backend->priv->children, source);
+
+	g_mutex_unlock (backend->priv->children_lock);
+
+	return removed;
+}
+
+static GList *
+collection_backend_children_list (ECollectionBackend *backend)
+{
+	GList *list, *link;
+
+	g_mutex_lock (backend->priv->children_lock);
+
+	list = g_hash_table_get_keys (backend->priv->children);
+
+	for (link = list; link != NULL; link = g_list_next (link))
+		g_object_ref (link->data);
+
+	g_mutex_unlock (backend->priv->children_lock);
+
+	return list;
+}
+
 static GFile *
 collection_backend_new_user_file (ECollectionBackend *backend)
 {
@@ -471,8 +520,9 @@ collection_backend_dispose (GObject *object)
 		g_object_unref (server);
 	}
 
-	while (!g_queue_is_empty (&priv->children))
-		g_object_unref (g_queue_pop_head (&priv->children));
+	g_mutex_lock (priv->children_lock);
+	g_hash_table_remove_all (priv->children);
+	g_mutex_unlock (priv->children_lock);
 
 	g_mutex_lock (priv->unclaimed_resources_lock);
 	g_hash_table_remove_all (priv->unclaimed_resources);
@@ -489,6 +539,9 @@ collection_backend_finalize (GObject *object)
 
 	priv = E_COLLECTION_BACKEND_GET_PRIVATE (object);
 
+	g_hash_table_destroy (priv->children);
+	g_mutex_free (priv->children_lock);
+
 	g_hash_table_destroy (priv->unclaimed_resources);
 	g_mutex_free (priv->unclaimed_resources_lock);
 
@@ -567,6 +620,35 @@ collection_backend_constructed (GObject *object)
 		(GDestroyNotify) g_object_unref);
 }
 
+static gboolean
+collection_backend_authenticate_sync (EBackend *backend,
+                                      ESourceAuthenticator *auth,
+                                      GCancellable *cancellable,
+                                      GError **error)
+{
+	ECollectionBackend *collection_backend;
+	ESourceRegistryServer *server;
+	EAuthenticationSession *session;
+	ESource *source;
+	const gchar *uid;
+	gboolean success;
+
+	source = e_backend_get_source (backend);
+	uid = e_source_get_uid (source);
+
+	collection_backend = E_COLLECTION_BACKEND (backend);
+	server = e_collection_backend_ref_server (collection_backend);
+	session = e_authentication_session_new (server, auth, uid);
+
+	success = e_source_registry_server_authenticate_sync (
+		server, session, cancellable, error);
+
+	g_object_unref (session);
+	g_object_unref (server);
+
+	return success;
+}
+
 static void
 collection_backend_populate (ECollectionBackend *backend)
 {
@@ -596,12 +678,9 @@ static void
 collection_backend_child_added (ECollectionBackend *backend,
                                 ESource *child_source)
 {
+	collection_backend_children_insert (backend, child_source);
 	collection_backend_bind_child_enabled (backend, child_source);
 
-	g_queue_push_tail (
-		&backend->priv->children,
-		g_object_ref (child_source));
-
 	/* Collection children are not removable. */
 	e_server_side_source_set_removable (
 		E_SERVER_SIDE_SOURCE (child_source), FALSE);
@@ -611,14 +690,132 @@ static void
 collection_backend_child_removed (ECollectionBackend *backend,
                                   ESource *child_source)
 {
-	if (g_queue_remove (&backend->priv->children, child_source))
-		g_object_unref (child_source);
+	collection_backend_children_remove (backend, child_source);
+}
+
+static gboolean
+collection_backend_create_resource_sync (ECollectionBackend *backend,
+                                         ESource *source,
+                                         GCancellable *cancellable,
+                                         GError **error)
+{
+	EAsyncClosure *closure;
+	GAsyncResult *result;
+	gboolean success;
+
+	closure = e_async_closure_new ();
+
+	e_collection_backend_create_resource (
+		backend, source, cancellable,
+		e_async_closure_callback, closure);
+
+	result = e_async_closure_wait (closure);
+
+	success = e_collection_backend_create_resource_finish (
+		backend, result, error);
+
+	e_async_closure_free (closure);
+
+	return success;
+}
+
+static void
+collection_backend_create_resource (ECollectionBackend *backend,
+                                    ESource *source,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+
+	simple = g_simple_async_result_new_error (
+		G_OBJECT (backend), callback, user_data,
+		G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+		_("%s does not support creating remote resources"),
+		G_OBJECT_TYPE_NAME (backend));
+
+	g_simple_async_result_complete_in_idle (simple);
+
+	g_object_unref (simple);
+}
+
+static gboolean
+collection_backend_create_resource_finish (ECollectionBackend *backend,
+                                           GAsyncResult *result,
+                                           GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	/* Assume success unless a GError is set. */
+	return !g_simple_async_result_propagate_error (simple, error);
+}
+
+static gboolean
+collection_backend_delete_resource_sync (ECollectionBackend *backend,
+                                         ESource *source,
+                                         GCancellable *cancellable,
+                                         GError **error)
+{
+	EAsyncClosure *closure;
+	GAsyncResult *result;
+	gboolean success;
+
+	closure = e_async_closure_new ();
+
+	e_collection_backend_delete_resource (
+		backend, source, cancellable,
+		e_async_closure_callback, closure);
+
+	result = e_async_closure_wait (closure);
+
+	success = e_collection_backend_delete_resource_finish (
+		backend, result, error);
+
+	e_async_closure_free (closure);
+
+	return success;
+}
+
+static void
+collection_backend_delete_resource (ECollectionBackend *backend,
+                                    ESource *source,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+
+	simple = g_simple_async_result_new_error (
+		G_OBJECT (backend), callback, user_data,
+		G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+		_("%s does not support deleting remote resources"),
+		G_OBJECT_TYPE_NAME (backend));
+
+	g_simple_async_result_complete_in_idle (simple);
+
+	g_object_unref (simple);
+}
+
+static gboolean
+collection_backend_delete_resource_finish (ECollectionBackend *backend,
+                                           GAsyncResult *result,
+                                           GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	/* Assume success unless a GError is set. */
+	return !g_simple_async_result_propagate_error (simple, error);
 }
 
 static void
 e_collection_backend_class_init (ECollectionBackendClass *class)
 {
 	GObjectClass *object_class;
+	EBackendClass *backend_class;
 
 	g_type_class_add_private (class, sizeof (ECollectionBackendPrivate));
 
@@ -629,10 +826,19 @@ e_collection_backend_class_init (ECollectionBackendClass *class)
 	object_class->finalize = collection_backend_finalize;
 	object_class->constructed = collection_backend_constructed;
 
+	backend_class = E_BACKEND_CLASS (class);
+	backend_class->authenticate_sync = collection_backend_authenticate_sync;
+
 	class->populate = collection_backend_populate;
 	class->dup_resource_id = collection_backend_dup_resource_id;
 	class->child_added = collection_backend_child_added;
 	class->child_removed = collection_backend_child_removed;
+	class->create_resource_sync = collection_backend_create_resource_sync;
+	class->create_resource = collection_backend_create_resource;
+	class->create_resource_finish = collection_backend_create_resource_finish;
+	class->delete_resource_sync = collection_backend_delete_resource_sync;
+	class->delete_resource = collection_backend_delete_resource;
+	class->delete_resource_finish = collection_backend_delete_resource_finish;
 
 	g_object_class_install_property (
 		object_class,
@@ -696,8 +902,15 @@ e_collection_backend_class_init (ECollectionBackendClass *class)
 static void
 e_collection_backend_init (ECollectionBackend *backend)
 {
+	GHashTable *children;
 	GHashTable *unclaimed_resources;
 
+	children = g_hash_table_new_full (
+		(GHashFunc) e_source_hash,
+		(GEqualFunc) e_source_equal,
+		(GDestroyNotify) g_object_unref,
+		(GDestroyNotify) NULL);
+
 	unclaimed_resources = g_hash_table_new_full (
 		(GHashFunc) g_str_hash,
 		(GEqualFunc) g_str_equal,
@@ -705,6 +918,8 @@ e_collection_backend_init (ECollectionBackend *backend)
 		(GDestroyNotify) g_object_unref);
 
 	backend->priv = E_COLLECTION_BACKEND_GET_PRIVATE (backend);
+	backend->priv->children = children;
+	backend->priv->children_lock = g_mutex_new ();
 	backend->priv->unclaimed_resources = unclaimed_resources;
 	backend->priv->unclaimed_resources_lock = g_mutex_new ();
 }
@@ -832,7 +1047,7 @@ e_collection_backend_list_calendar_sources (ECollectionBackend *backend)
 
 	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), NULL);
 
-	list = g_queue_peek_head_link (&backend->priv->children);
+	list = collection_backend_children_list (backend);
 
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		ESource *child_source = E_SOURCE (link->data);
@@ -841,6 +1056,8 @@ e_collection_backend_list_calendar_sources (ECollectionBackend *backend)
 				result_list, g_object_ref (child_source));
 	}
 
+	g_list_free_full (list, (GDestroyNotify) g_object_unref);
+
 	return g_list_reverse (result_list);
 }
 
@@ -873,7 +1090,7 @@ e_collection_backend_list_contacts_sources (ECollectionBackend *backend)
 
 	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), NULL);
 
-	list = g_queue_peek_head_link (&backend->priv->children);
+	list = collection_backend_children_list (backend);
 
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		ESource *child_source = E_SOURCE (link->data);
@@ -882,6 +1099,8 @@ e_collection_backend_list_contacts_sources (ECollectionBackend *backend)
 				result_list, g_object_ref (child_source));
 	}
 
+	g_list_free_full (list, (GDestroyNotify) g_object_unref);
+
 	return g_list_reverse (result_list);
 }
 
@@ -914,7 +1133,7 @@ e_collection_backend_list_mail_sources (ECollectionBackend *backend)
 
 	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), NULL);
 
-	list = g_queue_peek_head_link (&backend->priv->children);
+	list = collection_backend_children_list (backend);
 
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		ESource *child_source = E_SOURCE (link->data);
@@ -923,6 +1142,138 @@ e_collection_backend_list_mail_sources (ECollectionBackend *backend)
 				result_list, g_object_ref (child_source));
 	}
 
+	g_list_free_full (list, (GDestroyNotify) g_object_unref);
+
 	return g_list_reverse (result_list);
 }
 
+/**
+ * e_collection_backend_create_resource_sync
+ * @backend: an #ECollectionBackend
+ * @source: an #ESource
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @error: return location for a #GError, or %NULL
+ *
+ * Creates a server-side resource described by @source.  For example, if
+ * @source describes a new calendar, an equivalent calendar is created on
+ * the server.
+ *
+ * It is the implementor's responsibility to examine @source and determine
+ * what the equivalent server-side resource would be.  If this cannot be
+ * determined without ambiguity, the function must return an error.
+ *
+ * After the server-side resource is successfully created, the implementor
+ * must also add an #ESource to @backend's #ECollectionBackend:server.  This
+ * can either be done immediately or in response to some "resource created"
+ * notification from the server.  The added #ESource can be @source itself
+ * or a different #ESource instance that describes the new resource.
+ * implementor's responsibility to add an #ESource to.
+ *
+ * Since: 3.6
+ **/
+gboolean
+e_collection_backend_create_resource_sync (ECollectionBackend *backend,
+                                           ESource *source,
+                                           GCancellable *cancellable,
+                                           GError **error)
+{
+	ECollectionBackendClass *class;
+
+	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_val_if_fail (class->create_resource_sync != NULL, FALSE);
+
+	return class->create_resource_sync (
+		backend, source, cancellable, error);
+}
+
+void
+e_collection_backend_create_resource (ECollectionBackend *backend,
+                                      ESource *source,
+                                      GCancellable *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data)
+{
+	ECollectionBackendClass *class;
+
+	g_return_if_fail (E_IS_COLLECTION_BACKEND (backend));
+	g_return_if_fail (E_IS_SOURCE (source));
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_if_fail (class->create_resource != NULL);
+
+	class->create_resource (
+		backend, source, cancellable, callback, user_data);
+}
+
+gboolean
+e_collection_backend_create_resource_finish (ECollectionBackend *backend,
+                                             GAsyncResult *result,
+                                             GError **error)
+{
+	ECollectionBackendClass *class;
+
+	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_val_if_fail (class->create_resource_finish != NULL, FALSE);
+
+	return class->create_resource_finish (backend, result, error);
+}
+
+gboolean
+e_collection_backend_delete_resource_sync (ECollectionBackend *backend,
+                                           ESource *source,
+                                           GCancellable *cancellable,
+                                           GError **error)
+{
+	ECollectionBackendClass *class;
+
+	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_val_if_fail (class->delete_resource_sync != NULL, FALSE);
+
+	return class->delete_resource_sync (
+		backend, source, cancellable, error);
+}
+
+void
+e_collection_backend_delete_resource (ECollectionBackend *backend,
+                                      ESource *source,
+                                      GCancellable *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data)
+{
+	ECollectionBackendClass *class;
+
+	g_return_if_fail (E_IS_COLLECTION_BACKEND (backend));
+	g_return_if_fail (E_IS_SOURCE (source));
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_if_fail (class->delete_resource != NULL);
+
+	return class->delete_resource (
+		backend, source, cancellable, callback, user_data);
+}
+
+gboolean
+e_collection_backend_delete_resource_finish (ECollectionBackend *backend,
+                                             GAsyncResult *result,
+                                             GError **error)
+{
+	ECollectionBackendClass *class;
+
+	g_return_val_if_fail (E_IS_COLLECTION_BACKEND (backend), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+
+	class = E_COLLECTION_BACKEND_GET_CLASS (backend);
+	g_return_val_if_fail (class->delete_resource_finish != NULL, FALSE);
+
+	return class->delete_resource_finish (backend, result, error);
+}
+
