@@ -19,7 +19,9 @@
 
 #include <config.h>
 #include <glib/gi18n-lib.h>
-#include "libedataserver/e-categories.h"
+
+#include <libedataserver/libedataserver.h>
+
 #include "e-categories-selector.h"
 #include "e-data-server-ui-marshal.h"
 
@@ -381,6 +383,9 @@ e_categories_selector_set_items_checkable (ECategoriesSelector *selector,
 
 	g_return_if_fail (E_IS_CATEGORIES_SELECTOR (selector));
 
+	if ((selector->priv->checkable ? 1 : 0) == (checkable ? 1 : 0))
+		return;
+
 	selector->priv->checkable = checkable;
 
 	column = gtk_tree_view_get_column (
