@@ -30,14 +30,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
 
-#include <libedataserver/e-sexp.h>
-#include <libedataserver/e-categories.h>
-
-#include <libebackend/e-extensible.h>
-
-#include <libebook/e-book-client.h>
-#include <libebook/e-book-client-view.h>
-#include <libebook/e-book-query.h>
+#include <libebook/libebook.h>
+#include <libebackend/libebackend.h>
 
 #include "e-source-combo-box.h"
 #include "e-destination-store.h"
@@ -68,6 +62,7 @@ typedef struct {
 } SelData;
 
 struct _ENameSelectorDialogPrivate {
+	ESourceRegistry *registry;
 	ENameSelectorModel *name_selector_model;
 	GtkTreeModelSort *contact_sort;
 	GCancellable *cancellable;
@@ -87,6 +82,11 @@ struct _ENameSelectorDialogPrivate {
 	GtkSizeGroup *dest_label_size_group;
 };
 
+enum {
+	PROP_0,
+	PROP_REGISTRY
+};
+
 static void     search_changed                (ENameSelectorDialog *name_selector_dialog);
 static void     source_changed                (ENameSelectorDialog *name_selector_dialog, ESourceComboBox *source_combo_box);
 static void     transfer_button_clicked       (ENameSelectorDialog *name_selector_dialog, GtkButton *transfer_button);
@@ -146,12 +146,66 @@ name_selector_dialog_populate_categories (ENameSelectorDialog *name_selector_dia
 		G_CALLBACK (search_changed), name_selector_dialog);
 }
 
+static void
+name_selector_dialog_set_registry (ENameSelectorDialog *name_selector_dialog,
+                                   ESourceRegistry *registry)
+{
+	g_return_if_fail (E_IS_SOURCE_REGISTRY (registry));
+	g_return_if_fail (name_selector_dialog->priv->registry == NULL);
+
+	name_selector_dialog->priv->registry = g_object_ref (registry);
+}
+
+static void
+name_selector_dialog_set_property (GObject *object,
+                                   guint property_id,
+                                   const GValue *value,
+                                   GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_REGISTRY:
+			name_selector_dialog_set_registry (
+				E_NAME_SELECTOR_DIALOG (object),
+				g_value_get_object (value));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+name_selector_dialog_get_property (GObject *object,
+                                   guint property_id,
+                                   GValue *value,
+                                   GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_REGISTRY:
+			g_value_set_object (
+				value,
+				e_name_selector_dialog_get_registry (
+				E_NAME_SELECTOR_DIALOG (object)));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
 static void
 name_selector_dialog_dispose (GObject *object)
 {
+	ENameSelectorDialogPrivate *priv;
+
+	priv = E_NAME_SELECTOR_DIALOG_GET_PRIVATE (object);
+
 	remove_books (E_NAME_SELECTOR_DIALOG (object));
 	shutdown_name_selector_model (E_NAME_SELECTOR_DIALOG (object));
 
+	if (priv->registry != NULL) {
+		g_object_unref (priv->registry);
+		priv->registry = NULL;
+	}
+
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose (object);
 }
@@ -182,7 +236,7 @@ name_selector_dialog_constructed (GObject *object)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *cell_renderer;
 	GtkTreeSelection  *selection;
-	ESourceList       *source_list;
+	ESource *source;
 	gchar *tmp_str;
 	GtkWidget *name_selector_box;
 	GtkWidget *show_contacts_label;
@@ -210,6 +264,7 @@ name_selector_dialog_constructed (GObject *object)
 	GtkWidget *destination_box;
 	GtkWidget *status_message;
 	GtkWidget *source_combo;
+	const gchar *extension_name;
 
 	priv = E_NAME_SELECTOR_DIALOG_GET_PRIVATE (object);
 
@@ -358,13 +413,6 @@ name_selector_dialog_constructed (GObject *object)
 	g_object_unref (G_OBJECT (tmp_relation));
 	g_object_unref (G_OBJECT (tmp_relation_set));
 
-	/* Get addressbook sources */
-
-	if (!e_book_client_get_sources (&source_list, NULL)) {
-		g_warning ("ENameSelectorDialog can't find any addressbooks!");
-		return;
-	}
-
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (object))), name_selector_box, TRUE, TRUE, 0);
 
 	/* Store pointers to relevant widgets */
@@ -417,11 +465,11 @@ name_selector_dialog_constructed (GObject *object)
 
 	/* Create source menu */
 
-	source_combo = e_source_combo_box_new (source_list);
+	extension_name = E_SOURCE_EXTENSION_ADDRESS_BOOK;
+	source_combo = e_source_combo_box_new (priv->registry, extension_name);
 	g_signal_connect_swapped (
 		source_combo, "changed",
 		G_CALLBACK (source_changed), object);
-	g_object_unref (source_list);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (AddressBookLabel), source_combo);
 	gtk_widget_show (source_combo);
@@ -438,7 +486,10 @@ name_selector_dialog_constructed (GObject *object)
 
 	/* Display initial source */
 
-	/* TODO: Remember last used source */
+	source = e_source_registry_ref_default_address_book (priv->registry);
+	e_source_combo_box_set_active (
+		E_SOURCE_COMBO_BOX (source_combo), source);
+	g_object_unref (source);
 
 	/* Set up dialog defaults */
 
@@ -482,9 +533,23 @@ e_name_selector_dialog_class_init (ENameSelectorDialogClass *class)
 	g_type_class_add_private (class, sizeof (ENameSelectorDialogPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
+	object_class->set_property = name_selector_dialog_set_property;
+	object_class->get_property = name_selector_dialog_get_property;
 	object_class->dispose = name_selector_dialog_dispose;
 	object_class->finalize = name_selector_dialog_finalize;
 	object_class->constructed = name_selector_dialog_constructed;
+
+	g_object_class_install_property (
+		object_class,
+		PROP_REGISTRY,
+		g_param_spec_object (
+			"registry",
+			"Registry",
+			"Data source registry",
+			E_TYPE_SOURCE_REGISTRY,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -496,15 +561,29 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 
 /**
  * e_name_selector_dialog_new:
+ * @registry: an #ESourceRegistry
  *
  * Creates a new #ENameSelectorDialog.
  *
  * Returns: A new #ENameSelectorDialog.
  **/
 ENameSelectorDialog *
-e_name_selector_dialog_new (void)
+e_name_selector_dialog_new (ESourceRegistry *registry)
 {
-	return g_object_new (E_TYPE_NAME_SELECTOR_DIALOG, NULL);
+	g_return_val_if_fail (E_IS_SOURCE_REGISTRY (registry), NULL);
+
+	return g_object_new (
+		E_TYPE_NAME_SELECTOR_DIALOG,
+		"registry", registry, NULL);
+}
+
+ESourceRegistry *
+e_name_selector_dialog_get_registry (ENameSelectorDialog *name_selector_dialog)
+{
+	g_return_val_if_fail (
+		E_IS_NAME_SELECTOR_DIALOG (name_selector_dialog), NULL);
+
+	return name_selector_dialog->priv->registry;
 }
 
 /* --------- *
@@ -1024,7 +1103,6 @@ source_changed (ENameSelectorDialog *name_selector_dialog,
 	/* Start loading selected book */
 	e_client_utils_open_new (
 		source, E_CLIENT_SOURCE_TYPE_CONTACTS, TRUE, cancellable,
-		e_client_utils_authenticate_handler, parent,
 		book_loaded_cb, g_object_ref (name_selector_dialog));
 
 	g_object_unref (source);
