@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-#include <libebook/e-book-client.h>
+#include <libebook/libebook.h>
 
 #include "client-test-utils.h"
 
