@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 #include <stdlib.h>
-#include <libebook/e-book.h>
+#include <libebook/libebook.h>
 
 #include "client-test-utils.h"
 
@@ -44,7 +44,7 @@ add_contact (EBookClient *client)
 }
 
 static gboolean
-setup_book (EBookClient     **book_out)
+setup_book (EBookClient **book_out)
 {
 	GError *error = NULL;
 	gint   i, j;
@@ -147,7 +147,8 @@ finish_test (EBookClientView *view)
 }
 
 static void
-objects_added (EBookClientView *view, const GSList *contacts)
+objects_added (EBookClientView *view,
+               const GSList *contacts)
 {
 	const GSList *l;
 
@@ -171,7 +172,8 @@ objects_added (EBookClientView *view, const GSList *contacts)
 }
 
 static void
-objects_removed (EBookClientView *view, const GSList *ids)
+objects_removed (EBookClientView *view,
+                 const GSList *ids)
 {
 	const GSList *l;
 
@@ -181,7 +183,8 @@ objects_removed (EBookClientView *view, const GSList *ids)
 }
 
 static void
-complete (EBookClientView *view, const GError *error)
+complete (EBookClientView *view,
+          const GError *error)
 {
 	/* Now add a contact and assert that we received notification */
 	loading_view = FALSE;
@@ -217,7 +220,9 @@ setup_and_start_view (EBookClientView *view)
 }
 
 static void
-get_view_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
+get_view_cb (GObject *source_object,
+             GAsyncResult *result,
+             gpointer user_data)
 {
 	EBookClientView *view;
 	GError *error = NULL;
@@ -254,7 +259,8 @@ call_get_view (gpointer user_data)
 }
 
 gint
-main (gint argc, gchar **argv)
+main (gint argc,
+      gchar **argv)
 {
 	EBookClient *book_client;
 	EBookQuery *query;
